# tools to split audio data

import glob
from datetime import timedelta
from time import time

import numpy as np
import pandas as pd


def load_all_ratings(labels_dir, split_id_clip = False):
    '''
        loads all ratings
        - replaces sentiment by positive if sentiment > 0
          or by negative if sentiment < 0
        - replaces all emotions by 1 if > 0
        - adds none = 1 where all emotions + sentiment == 0
        labels_dir    : directory containing labels (ratings)
        split_id_clip : if true id and clip columns separate
                        if false single id_clip column
        returns       : DataFrame containing all ratings (3 per text/clip)
    '''
    label_files = glob.glob(f"{labels_dir}/*.csv")

    df_labels = []

    for filename in label_files:
        df = pd.read_csv(filename, index_col=None, header=0)
        df_labels.append(df)


    df_labels = pd.concat(df_labels, axis=0, ignore_index=True)

    label_cols = ['Input.VIDEO_ID', 'Input.CLIP',
              'Answer.anger', 'Answer.disgust',
              'Answer.fear', 'Answer.happiness',
              'Answer.sadness', 'Answer.surprise',
              'Answer.sentiment']

    label_new_cols = ['id', 'clip',
                      'anger', 'disgust',
                      'fear', 'happiness',
                      'sadness', 'surprise',
                      'sentiment']

    df_labels = df_labels[label_cols]
    df_labels.columns = label_new_cols

    # drop row all nan
    isna_idx = \
        df_labels.index[df_labels[df_labels.columns[2:]].isna().all(axis=1)]
    df_labels.drop(index=isna_idx, inplace=True)
    # replace remaining nan's with 0
    df_labels = df_labels.replace({np.nan : 0})
    # convert ratings to int
    df_labels[label_new_cols[2:]] = df_labels[label_new_cols[2:]].astype('Int64')
    # set emotions to 0 or 1
    df_labels[label_new_cols[2:-1]] = \
        df_labels[label_new_cols[2:-1]].applymap(lambda x : 1 if x > 0 else 0)

    # if sentiment > 0 convert to positive = 1, elif < 0 convert to negative = 1
    df_labels['positive'] = \
        df_labels['sentiment'].map(lambda x : 1 if x > 0 else 0)
    df_labels['negative'] = \
        df_labels['sentiment'].map(lambda x : 1 if x < 0 else 0)

    # drop sentiment column (now in positive/negative)
    df_labels.drop(columns='sentiment', inplace=True)

    # if none of emotion or sentiment == 1, set none to 1
    df_labels['none'] = 0
    none_idx = \
        df_labels[df_labels[df_labels.columns[2:]].sum(axis=1) == 0].index

    df_labels.loc[none_idx,'none'] = 1

    # remove '/' from id's
    df_labels['id'] = df_labels['id'].map(lambda x : str(x).split("/")[-1])
    if not split_id_clip:
        df_labels['id'] = df_labels['id'] + '_' + df_labels['clip'].astype(str)
        df_labels.drop(columns = 'clip', inplace = True)
        
        label_new_cols = ['id',
                          'none', 'positive', 'negative',
                          'anger', 'disgust',
                          'fear', 'happiness',
                          'sadness', 'surprise'
                         ]
    else:
        label_new_cols = ['id', 'clip',
                          'none', 'positive', 'negative',
                          'anger', 'disgust',
                          'fear', 'happiness',
                          'sadness', 'surprise'
                         ]
        
    df_labels = df_labels[label_new_cols]
    return df_labels


def get_labeled_clips(labels_dir):
    '''
        get id & clip number from labels
        labels_dir : directory containing raw labels
        returns   : dict of ids with correponding labeled clips
                    {'id' : [clip1, clip2, ... clip_n]}
    '''
    labeled_clips = {}
    labels = load_all_ratings(labels_dir, split_id_clip=True)

    labels = labels.drop_duplicates(subset=['id','clip'])

    uniq_ids = sorted(list(labels['id'].unique()))
    for i in uniq_ids:
        labeled_clips[i] = \
            sorted(
                labels[labels['id'] == i]['clip'].astype(int).to_list()
            )
    del labels
    return labeled_clips


def get_clips_info_from_text(text_dir, get_text=False):
    '''
        get audio information from text files to split audio into clips
        text_dir         : directory containing texts with audio info
        get_text         : add text column to info DataFrame
        returns: DataFrame
        - id (file id)
        - clip (clip number)
        - start_time (audio file segment start time)
        - end_time   (audio file segment end time)
        - len (audio clip len)
    '''
    text_files = glob.glob(f"{text_dir}/*.txt")
    dfs = []
    for file in text_files:
        with open(file, 'r') as text_file:
            lines = text_file.readlines()
            clips_info = []
            for line in lines:
                line_info = {}
                split_line = line.split('___')
                line_info['id'] = split_line[0]
                line_info['clip'] = int(split_line[1])
                line_info['start_time'] = round(abs(float(split_line[2])) ,3)
                line_info['end_time'] = round(float(split_line[3]), 3)
                line_info['len'] = \
                    line_info['end_time'] - line_info['start_time']

                if get_text:
                    line_info['text'] = split_line[4]
                
                clips_info.append(line_info)
        dfs.append(pd.DataFrame(clips_info))
    dfs = pd.concat(dfs, axis=0, ignore_index=True)
    return dfs


def get_labeled_clips_info(labels_dir, text_dir,
        get_text=False, show_progress=True):
    '''
        labels_dir       : directory containing labels
        text_dir         : directory containing text files and clip info
        get_text         : add text column to info DataFrame
        returns          : DataFrame containing:
                           id         : file id
                           clip       : audio clip/line number
                           start_time : audio clip start time
                           end time   : audio clip end time
                           len        : audio clip len
                           text       : text if get_text == True
                    clips_not_in_text (labeled_clips)
    '''

    stime = time()
    print("Recherche des ids et no. de clips annotés   ...")
    labeled_clips = get_labeled_clips(labels_dir)
    print("Extraction des infos audio (début/fin)      ...")
    clips_info = get_clips_info_from_text(text_dir, get_text)

    labeled_clips_info = []
    clips_not_in_text = []
    print("Selection de l'info audio des clips annotés ...")
    num_files = len(labeled_clips)
    files_processed = 0
    for i, clips in labeled_clips.items():
        mask_id = clips_info['id'] == i
        for clip in clips:
            mask_clip = clips_info['clip'] == clip
            clip_info = {}
            clip_info['id'] = i
            clip_info['clip'] = clip
            try:
                # if text with id contains clip, add info to clip_info
                len(clips_info[(mask_id) & (mask_clip)].index) == 1
                clip_info['start_time'] = \
                        clips_info[(mask_id) & (mask_clip)]['start_time'].values[0]
                clip_info['end_time'] = \
                        clips_info[(mask_id) & (mask_clip)]['end_time'].values[0]
                clip_info['len'] = \
                        clips_info[(mask_id) & (mask_clip)]['len'].values[0]
                if get_text:
                        clip_info['text'] = \
                            clips_info[(mask_id) & (mask_clip)]['text'].values[0]
                labeled_clips_info.append(clip_info)
            except:
                # if no clip in text add id and clip to errors
                clips_not_in_text.append(clip_info)

        if show_progress:
            files_processed += 1
            if files_processed % 10 == 0:
                print('.', end = '')
                if files_processed % 500 == 0:
                    print(f" {files_processed} de {num_files} fichiers")
    print("\nTemps d'exécution: ",
            f"{timedelta(seconds = round(time() - stime))} (h:mm:ss)")
    clips_not_in_text = pd.DataFrame(clips_not_in_text)
    labeled_clips_info = pd.DataFrame(labeled_clips_info)
    print(f"\n{labeled_clips_info.shape[0]} clips annotés")
    print(clips_not_in_text.shape[0],
        "clips annotés sans info audio")
    return labeled_clips_info, clips_not_in_text



# generate and train SVC model for audio sentiment detection


import pickle
from pathlib import Path

import pandas as pd
from sklearn.metrics import accuracy_score, confusion_matrix
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
from sklearn.svm import SVC

from emotion import module_dir, root_dir

DATA_DIR = Path(root_dir / "data/processed/audio")
FEATURES = Path(DATA_DIR / "audio_features.csv")
LABELS = Path(DATA_DIR / "sentiment_labels.csv")
ARTIFACTS_DIR = Path(module_dir / "artifacts")

def create_datasets(in_features, in_labels, test_size=0.2):
    '''
        - create aligned datasets with features and labels
        - split into train/test
        - fit StandardScaler on train features
        - scale train/test features
        in_features : pd.DataFrame containing audio clip features
                      (mean of each audio clips 40 mfcc's)
        in_labels   : sentiment labels for corresponding features
        test_size   : test data size
        returns     : X_train, X_test, y_train, y_test, scaler

    '''
    labels = in_labels.copy()
    features = in_features.copy().reindex(labels.index)
    X_train, X_test, y_train, y_test = \
        train_test_split(features, labels, test_size=test_size,
                         random_state=101,
                         stratify = labels.values.argmax(axis=1))
    scaler = StandardScaler().fit(X_train)
    X_train = scaler.transform(X_train)
    X_test = scaler.transform(X_test)

    print("\ntraining label count:")
    print(y_train.sum(axis=0))
    print("\ntest label count:")
    print(y_test.sum(axis=0))
    
    return X_train, X_test, y_train, y_test, scaler

def calc_metrics_per_class(y_true, y_pred, class_names=None):
    '''
        - calculate metrics per class (precision, recall, f1_score)
        - plus macro avergages
        y_true  : true labels
        y_pred  : predicted labels
        classes : class names

    '''
    if len(y_true.shape) > 1:
        conf_mtx = pd.DataFrame(
                            confusion_matrix(
                                y_true.values.argmax(axis=1),
                                y_pred.argmax(axis=1)
                            )
                        )
    else:
        conf_mtx = pd.DataFrame(
                            confusion_matrix(
                                y_true,
                                y_pred
                            )
                        )

    if class_names != None:
        conf_mtx.index = class_names
        conf_mtx.columns = class_names
    else:
        class_names = sorted(list(y_true.unique()))
        conf_mtx.index = class_names
        conf_mtx.columns = class_names

    class_metrics = {}
    for c in class_names:
        metrics = {}
        metrics['precision'] = \
            round(conf_mtx.loc[c, c] / conf_mtx.loc[:, c].sum(), 3)
        metrics['recall'] = \
            round(conf_mtx.loc[c, c] / conf_mtx.loc[c, :].sum(), 3)
        metrics['f1'] = \
            round(2 * (metrics['precision'] * metrics['recall'])/
                  (metrics['precision'] + metrics['recall']), 3)
        class_metrics[c] = metrics
    class_metrics = pd.DataFrame(class_metrics)
    macro_metrics = class_metrics.sum(axis=1) / 3
    class_metrics = class_metrics.T
    class_metrics.loc['macro'] = macro_metrics.round(3)
    return conf_mtx, class_metrics

def predict_show_metrics(model, X, y, show_confu=False, data_name='data'):
    '''
        predict class, calculate and show metrics
        model  : model
        X  : features matrix
        y  : labels matrix
        show_confu : display confusion matrix
        data_name  : name of data
    '''
    pred = model.predict(X)
    print(f"\n{data_name} accuracy : ",
          round(accuracy_score(y.values.argmax(axis=1), pred),3))

    conf_mtx, metrics = \
    calc_metrics_per_class(y.values.argmax(axis=1), pred,
                           class_names=y.columns.tolist())
    if show_confu:
        print("")
        print(conf_mtx)
    print("")
    print(metrics)

def train_svc(X_train, y_train, C=5):
    '''
        train SVM audio sentiment classifier
        X_train : training features
        y_train : training labels
        C       : regularization parameter
        returns : model
    '''
    print("\nTraining svc audio model ...")
    gamma='auto'
    svc = SVC(C=C, kernel='rbf', gamma=gamma, random_state=101)
    svc.fit(X_train, y_train.values.argmax(axis=1))
    return svc

def main():
    '''
        - load features/labels for model training
        - create train/test datasets
        - train model
        - save model
        - show train/test metrics
    '''
    if Path.is_file(FEATURES) and Path.is_file(LABELS):
        features = pd.read_csv(FEATURES, index_col = 0, header = 0)
        labels = pd.read_csv(LABELS, index_col = 0, header = 0)
        class_names = {c : v for c, v in enumerate(labels.columns.tolist())}
        X_train, X_test, y_train, y_test, scaler = \
            create_datasets(features, labels)

        audio_model = train_svc(X_train, y_train)
        audio_model.scaler = scaler
        audio_model.class_names = class_names
        predict_show_metrics(audio_model, X_train, y_train,
                data_name = "Train")

        predict_show_metrics(audio_model, X_test, y_test,
                data_name = "Test")
 
        with open(f"{ARTIFACTS_DIR}/audio_model.pkl", "wb") as f:
            pickle.dump(audio_model, f)
    else:
        print("Either features file or labels file missing from:\n",
            DATA_DIR
        )

if __name__ == "__main__":
    main()



# extract features from audio clips

# import os
import glob
import pathlib
import time
from datetime import timedelta
from pathlib import Path

import librosa
import numpy as np
import pandas as pd

from emotion import root_dir

# audio directories
AUDIO_DIR = Path(root_dir / "data/raw/audio")
AUDIO_CLIPS_DIR = Path(root_dir / "data/interim/audio")
AUDIO_FEATURES_DIR = Path(root_dir / "data/processed/audio")

# feature extraction parameters
AGG = 'mean'
N_MFCCS = 40
LEN_SECS = 3


def extract_features_mean(audio_file, len_secs=3,
                           n_mfccs=40, rms=False, zrc=False):
    '''
        extract mean features from audio clip (full or part)
        audio_file : wav file 16000 sampling rate, mono
        len_secs   : length of clip to use for median features
        n_mfccs    : number of mel frequency ceptral coefficients
        rms        : extract rms (energy)
        zrc        : extract zero crossing rate
        returns    : audio_features: pd.Series containing median audio features
                     feature_names : name of extracted features
    '''
    samples, srate = \
        librosa.load(audio_file, sr=None)
    
    if len_secs != 'full':
        num_samples = srate * len_secs
        if num_samples > len(samples):
            samples = samples[-num_samples:]
    
    audio_features = None
    feature_names = []
    if rms:
        feature_names += ['rms']
        audio_features = \
            np.mean(librosa.feature.rms(y=samples).T, axis=0)
    if zrc:
        feature_names += ['zrc']
        zrc = \
            np.mean(librosa.feature.zero_crossing_rate(y=samples).T,
                    axis=0)
        if isinstance(audio_features, np.ndarray):
            audio_features = np.append(audio_features, zrc)
        else:
            audio_features = zrc
    
    feature_names += ['mfcc_' + str(x) for x in range(1, n_mfccs + 1)]
    S = librosa.feature.melspectrogram(y=samples,
                                   sr=srate, n_mels=128, n_fft=1200,
                                   fmax=8000, win_length=1200, hop_length=800)
    mfccs = librosa.feature.mfcc(S=librosa.power_to_db(S), n_mfcc=n_mfccs)
    mfccs = np.mean(mfccs, axis=1)
    if isinstance(audio_features, np.ndarray):
        audio_features = np.append(audio_features, mfccs)
    else:
        audio_features = mfccs
    return audio_features, feature_names


def extract_features_median(audio_file, len_secs=3,
                           n_mfccs=40, rms=False, zrc=False):
    '''
        extract median features from audio clip (full or part)
        audio_file : wav file 16000 sampling rate, mono
        len_secs   : length of clip to use for median features
        n_mfccs    : number of mel frequency ceptral coefficients
        rms        : extract rms (energy)
        zrc        : extract zero crossing rate
        returns    : audio_features: pd.Series containing median audio features
                     feature_names : name of extracted features
    '''
    samples, srate = \
        librosa.load(audio_file, sr=None)
    
    if len_secs != 'full':
        num_samples = srate * len_secs
        if num_samples > len(samples):
            samples = samples[-num_samples:]
    
    audio_features = None
    feature_names = []
    if rms:
        feature_names += ['rms']
        audio_features = \
            np.median(librosa.feature.rms(y=samples).T, axis=0)
    if zrc:
        feature_names += ['zrc']
        zrc = \
            np.median(librosa.feature.zero_crossing_rate(y=samples).T,
                    axis=0)
        if isinstance(audio_features, np.ndarray):
            audio_features = np.append(audio_features, zrc)
        else:
            audio_features = zrc
    
    feature_names += ['mfcc_' + str(x) for x in range(1, n_mfccs + 1)]
    S = librosa.feature.melspectrogram(y=samples,
                                   sr=srate, n_mels=64, #128,
                                   fmax=8000, hop_length=512)
    mfccs = librosa.feature.mfcc(S=librosa.power_to_db(S), n_mfcc=n_mfccs)
    mfccs = np.median(mfccs, axis=1)
    if isinstance(audio_features, np.ndarray):
        audio_features = np.append(audio_features, mfccs)
    else:
        audio_features = mfccs
    return audio_features, feature_names


def extract_features_from_files(files, agg='mean', len_secs=3, n_mfccs=40,
        rms=False, zrc=False, show_progress=True):
    '''
        extract audio features from files
        files  : directory containing wav files 16000 sampling rate, mono
        file_names : list of file names to extract features from
        agg        : type of aggregation to perform on features
                     'median' or 'mean'
        len_secs   : length of audio clip on which to extract features
        n_mfccs    : number of mel frequency ceptral coefficients
        rms        : extract rms (energy)
        zrc        : extract zero crossing rate
        returns    : audio_features: pd.Series containing median audio features
    '''
    audio_features = {}
    stime = time.time()
    num_files = len(files)
    for i, f in enumerate(files):
        if agg == 'median':
            clip_features, fnames = \
                extract_features_median(f, len_secs, n_mfccs, rms, zrc)
        else:
            clip_features, fnames = \
                extract_features_mean(f, len_secs, n_mfccs, rms, zrc)
        audio_features[str(i)] = clip_features

        if show_progress:
            if i % 10 == 0 and i != 0:
                print('.', end = '')
                if i % 500 == 0:
                    print(f" {i} de {num_files} fichiers")

    audio_features = pd.DataFrame(audio_features).T
    audio_features.columns = fnames
    etime = time.time()
    proc_time = timedelta(seconds = round(etime - stime))
    if show_progress:
        print("\n{audio_features.shape[0]}",
            f"fichiers extraits: {proc_time} (h:mm:ss)")
    return audio_features

def extract_features_from_dir(audio_dir, file_names=None,
                                    agg='mean', len_secs=3,
                                    n_mfccs=40, rms=False, zrc=False,
                                    show_progress=True):
    '''
        extract audio features from directory,
         all if file_names == None, files in file_names if not None
        audio_dir  : directory containing wav files 16000 sampling rate, mono
        file_names : list of file names to extract features from
                     file_id of form fileid_clipnumber without .wav
        agg        : type of aggregation to perform on features
                     'median' or 'mean'
        len_secs   : length of audio clip on which to extract features
        n_mfccs    : number of mel frequency ceptral coefficients
        rms        : extract rms (energy)
        zrc        : extract zero crossing rate
        returns    : audio_features: pd.Series containing median audio features
    '''
    audio_features = {}
    stime = time.time()
    if file_names is None:
        file_names = glob.glob(f"{audio_dir}/*.wav")
    else:
        file_names = [f"{audio_dir}/{f}.wav" for f in file_names]
    num_files = len(file_names)
    for i, f in enumerate(file_names):
        if agg == 'median':
            clip_features, fnames = \
                extract_features_median(f, len_secs, n_mfccs, rms, zrc)
        else:
            clip_features, fnames = \
                extract_features_mean(f, len_secs, n_mfccs, rms, zrc)
        clip_id = f.rsplit('.', maxsplit = 1)[0].rsplit('/', maxsplit = 1)[-1]
        audio_features[clip_id] = clip_features

        if show_progress:
            if i % 10 == 0 and i != 0:
                print('.', end = '')
                if i % 500 == 0:
                    print(f" {i} de {num_files} fichiers")

    audio_features = pd.DataFrame(audio_features).T
    audio_features.columns = fnames
    etime = time.time()
    proc_time = timedelta(seconds = round(etime - stime))
    if show_progress:
        print("\n{audio_features.shape[0]}",
            f"fichiers extraits: {proc_time} (h:mm:ss)")
    return audio_features

def main():
    feature_file = \
        Path(AUDIO_FEATURES_DIR / \
            "audio_features.csv")
    if pathlib.Path.is_file(feature_file):
        print(f"{feature_file} already exists, no export to csv")
    else:
        features = \
            extract_features_from_dir(AUDIO_CLIPS_DIR, file_names=None,
                    agg=AGG, len_secs=LEN_SECS, n_mfccs=N_MFCCS
            )
        features.to_csv(feature_file,
                header=True, index=True)
        print(f"features exported to {feature_file}")

if __name__ == "__main__":
    main()




import numpy as np
import pandas as pd
import tensorflow as tf
import tensorflow_text
import tensorflow_hub as hub


from pathlib import Path
from emotion import module_dir, root_dir
from tensorflow.keras.models import model_from_json
from transformers import BertTokenizerFast, TFBertModel
from sklearn.model_selection import train_test_split as tts

from emotion import module_dir, root_dir

MAX_LEN = 20


module_url = 'https://tfhub.dev/tensorflow/bert_en_uncased_L-12_H-768_A-12/2'
BERT_LAYER = hub.KerasLayer(module_url, trainable=True)
TOKENIZER = BertTokenizerFast.from_pretrained('bert-base-uncased')
DATA_DIR = Path(root_dir / 'data/processed/text')
FEATURES = Path(DATA_DIR / 'polarity.csv')

ARTIFACTS_DIR = Path(module_dir / 'artifacts')


#Interesting/meaningful metrics
METRICS = [
    tf.keras.metrics.CategoricalAccuracy(name = 'accuracy'),
    tf.keras.metrics.Precision(name = 'precision'),
    tf.keras.metrics.Recall(name = 'recall')
]


#Encodes lists of texts into BERT-useable tensors
def bert_encode(texts, tokenizer=TOKENIZER, max_len=MAX_LEN):
    all_tokens = []
    all_masks = []
    all_segments = []
    
    for text in texts:
        text = tokenizer.tokenize(text)
            
        text = text[:max_len-2]
        input_sequence = ["[CLS]"] + text + ["[SEP]"]
        pad_len = max_len - len(input_sequence)
        
        tokens = tokenizer.convert_tokens_to_ids(input_sequence) + [0] * pad_len
        pad_masks = [1] * len(input_sequence) + [0] * pad_len
        segment_ids = [0] * max_len
        
        all_tokens.append(tokens)
        all_masks.append(pad_masks)
        all_segments.append(segment_ids)
    
    return np.array(all_tokens), np.array(all_masks), np.array(all_segments)


#Building a neural-net that uses BERT embeddings
def build_model(bert_layer=BERT_LAYER, max_len=MAX_LEN):
    input_word_ids = tf.keras.Input(shape=(max_len,), dtype=tf.int32, name="input_word_ids")
    input_mask = tf.keras.Input(shape=(max_len,), dtype=tf.int32, name="input_mask")
    segment_ids = tf.keras.Input(shape=(max_len,), dtype=tf.int32, name="segment_ids")

    pooled_output, sequence_output = bert_layer([input_word_ids, input_mask, segment_ids])
    clf_output = sequence_output[:, 0, :]

    #net = tf.keras.layers.Dense(64, activation='relu')(clf_output)
    #net = tf.keras.layers.Dropout(0.2)(net)
    net = tf.keras.layers.Dense(32, activation='relu')(clf_output)
    net = tf.keras.layers.Dropout(0.2)(net)
    out = tf.keras.layers.Dense(3, activation='softmax')(net)
    
    model = tf.keras.models.Model(inputs=[input_word_ids, input_mask, segment_ids], outputs=out)
    model.compile(tf.keras.optimizers.Adam(learning_rate=1e-5), loss='categorical_crossentropy', metrics=METRICS)
    
    return model

#Simple training function with a few parameters
def train_dnn(model, X_train, y_train, e=4):
    checkpoint = tf.keras.callbacks.ModelCheckpoint('checkpoint_model.h5', monitor='val_accuracy', save_best_only=True, verbose=1)
    earlystopping = tf.keras.callbacks.EarlyStopping(monitor='val_accuracy', patience=5, verbose=1)

    train_input = bert_encode(X_train)
    train_labels = y_train

    train_history = model.fit(
            train_input, train_labels, 
            validation_split=0.2,
            epochs=e,
            callbacks=[checkpoint, earlystopping],
            batch_size=4,
            verbose=1
            )
    return train_history


def main():
    if Path.is_file(FEATURES):
        data = pd.read_csv(FEATURES)
        x = data.clean_text.values
        dummy_sents = pd.get_dummies(data.sentiment)
        y = dummy_sents.values
        X_train, X_test, y_train, y_test = tts(x, y, test_size = 0.1)
        model = build_model()

        train_dnn(model, X_train, y_train)

        model_json = model.to_json()
        with open(f"{ARTIFACTS_DIR}/text_model.json", "w") as json_file:
            json_file.write(model_json)
        
        model.save_weights(f"{ARTIFACTS_DIR}/weights.h5")

    else:
        print(f"MISSING FILES FROM {DATA_DIR}")



if __name__ == "__main__":
    main()


import os
from pathlib import Path

import numpy as np
#import tensorflow as tf
#import tensorflow_text
#import tensorflow_hub as hub
#from tensorflow.keras.models import model_from_json
from emotion import module_dir, root_dir
"""
from emotion.features.text.extract_text import (
    clean_punct_digits,
    remove_nonascii,
    remove_stamps_str
)
"""


ARTIFACTS_DIR = Path(module_dir / "artifacts")

MODEL = f"{ARTIFACTS_DIR}/text_model.json"
WEIGHTS =f"{ARTIFACTS_DIR}/weights.h5"

class TextModel():
    '''
    Deep neural network classifier using BERT embeddings
    '''
    def __init__(self, model=None, weights=None):
        self.model = 'dummy_model'
           # self._model = model_from_json(model and open(model, 'r').read() or open(MODEL, 'r').read(), custom_objects={'KerasLayer':hub.KerasLayer})
           # self._model.load_weights(weights or WEIGHTS)
            

    def preprocess(self, texts):
        #cleaned_list = [clean_punct_digits(remove_nonascii(text)) for text in texts]
        #encoding = bert_encode(cleaned_list)
        encoding = [np.random.randint(3) for i in range(len(texts))]
        return encoding

    # Converts the classes to their assigned sentiment
    def to_sentiment(self, proba_list):
        sent_dict = {0 : 'negative', 
                     1 : 'neutral',
                     2 : 'positive'}

        return [sent_dict[proba] for proba in proba_list]

    # Makes the prediction on encoding
    def predict(self, encoding):
        sents = self.to_sentiment(encoding)
        #sents = self.to_sentiment([np.argmax(pred) for pred in preds])
        return sents


from pathlib import Path

import pandas as pd
from emotion import root_dir
from emotion.utils import create_csv

LABELS_DIR = Path(root_dir / "data/raw/labels")
CSV_FILES = LABELS_DIR.iterdir()
EMOTIONS = ["anger", "disgust", "fear", "happiness", "sadness", "surprise"]
EMOTIONS_COLS = [f"Answer.{emotion}" for emotion in EMOTIONS]
SENTIMENT_COL = "Answer.sentiment"
SENTIMENT = "sentiment"


def merge_intensity(df: pd.DataFrame) -> pd.DataFrame:
    """
    Use 0 or 1 for presence of emotion and -1, 0 or 1 for polarity.

    Parameters
    ----------
    df : pd.DataFrame
        The DataFrame to transform.

    Returns
    -------
    pd.DataFrame

    """
    return pd.concat(
        [
            df[EMOTIONS].applymap(lambda v: v > 0 and 1 or 0),
            df[SENTIMENT].map(lambda v: v < 0 and -1 or v > 0 and 1 or 0),
        ],
        axis=1,
    )


def merge_agreement(by: str, df: pd.DataFrame) -> pd.DataFrame:
    """
    Select only segments from which there is at most a disagreeement of 1 ordinality
    and take the median/mode as the 'winning' modality.

    Parameters
    ----------
    by: str
        column to group by.
    df : pd.DataFrame
        DataFrame to process.

    Returns
    -------
    pd.DataFrame
        The transformed DataFrame.
    """
    groupped = df.groupby(by=by)
    all_modalities = groupped[EMOTIONS + [SENTIMENT]]

    return all_modalities.median()[(all_modalities.std(0) < 0.48).all(axis=1)]


def main():
    dfs = {csv_file.stem: pd.read_csv(csv_file) for csv_file in CSV_FILES}

    # Standardize 'Input.VIDEO_ID' for `pom_extra_sqa_mono_results` dataset
    dfs["pom_extra_sqa_mono_results"]["Input.VIDEO_ID"] = dfs[
        "pom_extra_sqa_mono_results"
    ]["Input.VIDEO_ID"].map(lambda x: x.split("/")[1])

    # Merge all dataframes
    df_init = pd.concat(dfs.values())

    # Relevant columns
    ids = (
        df_init["Input.VIDEO_ID"].astype(str) + "_" + df_init["Input.CLIP"].astype(str)
    )
    ids.rename("id", inplace=True)
    df = pd.concat([ids, df_init[EMOTIONS_COLS + [SENTIMENT_COL]]], axis=1)
    df = df.rename(
        columns=dict(zip(EMOTIONS_COLS, EMOTIONS), **{SENTIMENT_COL: "sentiment"})
    )
    df = df.set_index("id")

    # Drop missing values
    missing = df[df.isnull().any(axis=1)]
    df = df.drop(index=missing.index)

    df_merged_intensity = merge_intensity(df)
    df_merged_agreement = merge_agreement("id", df_merged_intensity).astype(int)

    create_csv(root_dir / "data/interim/labels" / "labels.csv", df_merged_agreement)

    return df_merged_agreement


if __name__ == "__main__":
    main()
